/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.albumtab;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.gui.ImagePreview;
import com.googlepages.dronten.jtagger.model.GenreModel;
import com.googlepages.dronten.jtagger.model.PictureModel;
import com.googlepages.dronten.jtagger.renamer.AlbumRenamer;
import com.googlepages.dronten.jtagger.util.IOUtils;
import com.googlepages.dronten.jtagger.util.Log;
import com.googlepages.dronten.jtagger.util.Pref;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;


/**
 * Display all rename album options.
 *
 * @see AlbumRenamer
 */
public class AlbumOptionsPanel extends JPanel implements ActionListener {
    private static final long           serialVersionUID = 1;
    private static String               GENRE2_FIRST_STRING = "Keep Genre for ID3v2";

    private JCheckBox                   aArtistCheck;
    private JTextField                  aArtistInput;
    private JCheckBox                   aAlbumCheck;
    private JTextField                  aAlbumInput;
    private JComboBox                   aGenreCombo;
    private JComboBox                   aGenreCombo2;
    private JCheckBox                   aTrackCheck;
    private JTextField                  aTrackInput;
    private JCheckBox                   aCommentCheck;
    private JTextField                  aCommentInput;
    private JCheckBox                   aYearCheck;
    private JTextField                  aYearInput;
    private JCheckBox                   aCopyrightCheck;
    private JTextField                  aCopyrightInput;
    private JCheckBox                   aComposerCheck;
    private JTextField                  aComposerInput;
    private JCheckBox                   aEncodedByCheck;
    private JTextField                  aEncodedByInput;
    private JCheckBox                   aPictureCheck;
    private JTextField                  aPictureDescription;
    private PictureModel                aPictureModel;
    private JButton                     aOpenFileButton;
    private JButton                     aResetButton;
    private JFileChooser                aFileChooser;


    /**
     * Create rename options panel.<br>
     * It has all gui components for rename options for album tags.<br>
     */
    public AlbumOptionsPanel() {
        super();
        JPanel top = new JPanel();
        JPanel bottom = new JPanel();

        top.setLayout(new BoxLayout(top, BoxLayout.Y_AXIS));
        bottom.setLayout(new BoxLayout(bottom, BoxLayout.Y_AXIS));

        setLayout(new BorderLayout());
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        /*********************************************************************/

        aArtistCheck = ComponentFactory.createCheck(false, "Set Artist", "Enter artist text to use");
        aArtistInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aArtistCheck, aArtistInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aAlbumCheck = ComponentFactory.createCheck(false, "Set Album", "Enter album text to use");
        aAlbumInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aAlbumCheck, aAlbumInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aGenreCombo = ComponentFactory.createCombo(new GenreModel("Keep Genre for ID3v1"), 0, "<html>Select new genre for ID3v1 or keep existing genre<br>ID3v 1.1 tag checkbox must be set in the setup</html>", 0, 0);
        top.add(ComponentFactory.createOnePanel(aGenreCombo));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aGenreCombo2 = ComponentFactory.createCombo(new GenreModel(GENRE2_FIRST_STRING), 0, "<html>Select new genre or enter genre string for ID3v2 or keep existing genre<br>ID3v 2.3 tag checkbox must be set in the setup</html>", 0, 0);
        aGenreCombo2.setEditable(true);
        top.add(ComponentFactory.createOnePanel(aGenreCombo2));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aYearCheck = ComponentFactory.createCheck(false, "Set Year", "Enter year to use");
        aYearInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aYearCheck, aYearInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aTrackCheck = ComponentFactory.createCheck(false, "Set Track Number", "<html>Enter start track number<br>Default is 1</html>");
        aTrackInput = ComponentFactory.createInput("1", "");
        top.add(ComponentFactory.createTwoPanel(aTrackCheck, aTrackInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aCommentCheck = ComponentFactory.createCheck(false, "Set Comment", "Enter comment text to use");
        aCommentInput = ComponentFactory.createInput("", "");
        top.add(ComponentFactory.createTwoPanel(aCommentCheck, aCommentInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aCopyrightCheck = ComponentFactory.createCheck(false, "Set Copyright", "Enter copyright text to use");
        aCopyrightInput = ComponentFactory.createInput("", "<html>This tag doesn't exist in MP3 ID3v 1 tags!<br>Set year and then space and copyright message.<br><b>YYYY Copyrightstring</b></html>");
        top.add(ComponentFactory.createTwoPanel(aCopyrightCheck, aCopyrightInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aComposerCheck = ComponentFactory.createCheck(false, "Set Composer", "Enter composer text to use");
        aComposerInput = ComponentFactory.createInput("", "This tag doesn't exist in MP3 ID3v 1 tags!");
        top.add(ComponentFactory.createTwoPanel(aComposerCheck, aComposerInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aEncodedByCheck = ComponentFactory.createCheck(false, "Set Encoded By", "Enter 'encoded by' text to use");
        aEncodedByInput = ComponentFactory.createInput("", "This tag doesn't exist in MP3 ID3v 1 tags!");
        top.add(ComponentFactory.createTwoPanel(aEncodedByCheck, aEncodedByInput));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        /*********************************************************************/

        top.add(Box.createRigidArea(new Dimension(0, 10)));
        top.add(new JSeparator());
        top.add(Box.createRigidArea(new Dimension(0, 10)));

        /*********************************************************************/

        aPictureCheck = ComponentFactory.createCheck(false, "Set Picture", "<html>Select the picture to add or update<br>To remove pictures check the box and use no image</html>");
        aPictureDescription = ComponentFactory.createInput("", "Image description");
        top.add(ComponentFactory.createTwoPanel(aPictureCheck, aPictureDescription));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aPictureModel = new PictureModel();
        aPictureModel.setPictureLabel(new JLabel());
        aPictureModel.getPictureLabel().setIcon(JTagger.NO_IMAGE);
        top.add(ComponentFactory.createOnePanel(aPictureModel.getPictureLabel()));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        aOpenFileButton = ComponentFactory.createButton("Load Picture...", "Load and set the first picture in the audio track", this);
        top.add(ComponentFactory.createOnePanel(aOpenFileButton));
        top.add(Box.createRigidArea(new Dimension(0, 5)));

        /*********************************************************************/

        aResetButton = ComponentFactory.createButton("Reset &Options", "Reset and clear all options", this);
        bottom.add(aResetButton);

        /*********************************************************************/

        add(top, BorderLayout.NORTH);
        add(bottom, BorderLayout.SOUTH);

        setMinimumSize(new Dimension(150, Short.MAX_VALUE));
        setPreferredSize(new Dimension(250, Short.MAX_VALUE));

        String path = Pref.getPref(Constants.PREF_LASTDDIR_PICTURE, "");

        aFileChooser = new JFileChooser();
        aFileChooser.setCurrentDirectory(new File(path));
        aFileChooser.setFileFilter(new FileFilter(){

            @Override
            public boolean accept(File f) {
                return f.getName().endsWith(".jpg")||f.getName().endsWith(".gif")||f.getName().endsWith(".png")||f.isDirectory();
            }

            @Override
            public String getDescription() {
                return "JPG & GIF & PNG Images";
            }

        });

        aFileChooser.setAccessory(new ImagePreview(aFileChooser));
    }


    /**
     * Reset options.
     *
     * @param actionEvent
     */
    @Override public void actionPerformed(ActionEvent actionEvent) {
        if (actionEvent.getSource() == aResetButton) {
            aArtistCheck.setSelected(false);
            aArtistInput.setText("");
            aAlbumCheck.setSelected(false);
            aAlbumInput.setText("");
            aGenreCombo.setSelectedIndex(0);
            aGenreCombo2.setSelectedIndex(0);
            aTrackCheck.setSelected(false);
            aTrackInput.setText("1");
            aYearCheck.setSelected(false);
            aYearInput.setText("");
            aCommentCheck.setSelected(false);
            aCommentInput.setText("");
            aCopyrightCheck.setSelected(false);
            aCopyrightInput.setText("");
            aComposerCheck.setSelected(false);
            aComposerInput.setText("");
            aEncodedByCheck.setSelected(false);
            aEncodedByInput.setText("");
            aPictureCheck.setSelected(false);
            aPictureDescription.setText("");
            aPictureModel.getPictureLabel().setIcon(JTagger.NO_IMAGE);
            aPictureModel.setPictureData(null);
        }
        else if (actionEvent.getSource() == aOpenFileButton) {
            try {
                if (aFileChooser.showOpenDialog(AlbumOptionsPanel.this) == JFileChooser.APPROVE_OPTION) {
                    File file = aFileChooser.getSelectedFile();

                    if (file != null){
                        ImageIcon icon = new ImageIcon(file.toURI().toURL());
                        icon = new ImageIcon(icon.getImage().getScaledInstance(110, 110, Image.SCALE_DEFAULT));
                        aPictureModel.getPictureLabel().setIcon(icon);
                        aPictureModel.setPictureData(IOUtils.toByteArray(file));
                        aPictureModel.setMimeType(IOUtils.getMimeTypeFromFile(file.getName()));
                        Log.get().addTime(1,"Loaded image: " + file.getName());
                    }
                }

                Pref.setPref(Constants.PREF_LASTDDIR_PICTURE, aFileChooser.getCurrentDirectory().getAbsolutePath());
            }
            catch (Exception e) {
                JTagger.get().setMessage(e.getMessage(), true);
                Log.get().addTime(1, e.getMessage());
            }
        }
    }


    /**
     * Create renamer object with all options set using the gui controls.
     *
     * @return AlbumRenamer object.
     */
    public AlbumRenamer getRenamer() {
        AlbumRenamer renamer = new AlbumRenamer();

        if (aArtistCheck.isSelected()) {
            renamer.aUseArtist = aArtistInput.getText();
        }

        if (aAlbumCheck.isSelected()) {
            renamer.aUseAlbum = aAlbumInput.getText();
        }

        if (aGenreCombo.getSelectedIndex() > 0) {
            renamer.aUseGenre = (String) aGenreCombo.getSelectedItem();
        }

        String s = (String) aGenreCombo2.getSelectedItem();
        if (s.equalsIgnoreCase(GENRE2_FIRST_STRING) == false) {
            renamer.aUseGenre2 = s;
        }

        if (aYearCheck.isSelected()) {
            renamer.aUseYear = aYearInput.getText();
        }

        if (aTrackCheck.isSelected()) {
            renamer.aUseTrack = aTrackInput.getText();
        }

        if (aCommentCheck.isSelected()) {
            renamer.aUseComment = aCommentInput.getText();
        }

        if (aCopyrightCheck.isSelected()) {
            renamer.aUseCopyright = aCopyrightInput.getText();
        }

        if (aComposerCheck.isSelected()) {
            renamer.aUseComposer = aComposerInput.getText();
        }

        if (aEncodedByCheck.isSelected()) {
            renamer.aUseEncodedBy = aEncodedByInput.getText();
        }

        if (aPictureCheck.isSelected()) {
            renamer.aPictureModel = aPictureModel;
            renamer.aPictureDescription = aPictureDescription.getText();
        }

        return renamer;
    }
}
