/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.dlg.setup;

import com.googlepages.dronten.jtagger.Constants;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.util.Pref;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * Set preference for mp3 tracks.
 */
public class MP3Setup extends BaseSetup implements ActionListener {
    private static final long   serialVersionUID = 1;
    private JCheckBox           aEncoding = null;
    private JCheckBox           aID3v1 = null;
    private JCheckBox           aID3v2 = null;
    private JCheckBox           aUsePic = null;


    /**
     *
     */
    public MP3Setup() {
        super();

        String help = "<html>Beware that ID3v 1.1 tags can't use 'UTF-8' encodings!<br>";
        help += "So if the tags have none 'Latin-1' characters do turn of 'UTF-8' encoding or don't use ID3v 1.1 tags.</html>";

        aEncoding = ComponentFactory.createCheck(Pref.getPref(Constants.PREF_UNICODE_KEY, Constants.PREF_UNICODE_DEFAULT).length() > 0, "Use UTF-8 encoding", help, 0, 0);
        aID3v1 = ComponentFactory.createCheck(Pref.getPref(Constants.PREF_ID3V1_KEY, Constants.PREF_ID3V1_DEFAULT).length() > 0, "Write ID3v 1.1", "Write tags in the ID3v 1.1 format", 0, 0);
        aID3v2 = ComponentFactory.createCheck(Pref.getPref(Constants.PREF_ID3V2_KEY, Constants.PREF_ID3V2_DEFAULT).length() > 0, "Write ID3v 2.3", "Write tags in the ID3v 2.3 format", 0, 0);
        aUsePic = ComponentFactory.createCheck(Pref.getPref(Constants.PREF_USEPIC_KEY, Constants.PREF_USEPIC_DEFAULT).length() > 0, "Use Images", "Use and keep images in ID3v 2 format", 0, 0);

        add(ComponentFactory.createOnePanel(aEncoding));
        add(Box.createRigidArea(new Dimension(0, 5)));
        add(ComponentFactory.createOnePanel(aID3v1));
        add(Box.createRigidArea(new Dimension(0, 5)));
        add(ComponentFactory.createOnePanel(aID3v2));
        add(Box.createRigidArea(new Dimension(0, 5)));
        add(ComponentFactory.createOnePanel(aUsePic));
    }


    /**
     * @param actionEvent
     */
    @Override public void actionPerformed(ActionEvent actionEvent) {
    }


    /**
     * Save settings.
     */
    @Override public void save() {
        try {
            Pref.setPref(Constants.PREF_UNICODE_KEY, aEncoding.isSelected() ? "YES" : Constants.PREF_UNICODE_DEFAULT);
            Pref.setPref(Constants.PREF_ID3V1_KEY, aID3v1.isSelected() ? Constants.PREF_ID3V1_DEFAULT : "");
            Pref.setPref(Constants.PREF_ID3V2_KEY, aID3v2.isSelected() ? Constants.PREF_ID3V2_DEFAULT : "");
            Pref.setPref(Constants.PREF_USEPIC_KEY, aUsePic.isSelected() ? Constants.PREF_USEPIC_DEFAULT : "");
        }
        catch (Exception e) {
        }
    }
}
