/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.titletab;

import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.model.TitleModel;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;


/**
 * List data in a table that are synced to the TrackModel object.<br>
 *
 * @see com.googlepages.dronten.jtagger.model.TrackModel
 */
public class TitleTablePanel extends JPanel {
    private static final long   serialVersionUID = 1;
    public JButton              aTestRenameButton = null;
    public JButton              aUndoTestButton = null;
    public JButton              aUpdateButton = null;
    private JTable              aTable = null;
    private JPanel              aButtonPanel = null;


    /**
     *
     */
    public TitleTablePanel() {
        super();
        setLayout(new BorderLayout());
        aTable = new JTable(new TitleModel());
        JScrollPane centerPane = new JScrollPane();
        JPanel southPanel = new JPanel();

        setBorder(BorderFactory.createEmptyBorder(5, 3, 5, 3));
        southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.Y_AXIS));

        aTable.getColumnModel().getColumn(0).setMinWidth(50);
        aTable.getColumnModel().getColumn(0).setPreferredWidth(50);
        aTable.getColumnModel().getColumn(0).setMaxWidth(100);
        aTable.getColumnModel().getColumn(1).setMinWidth(50);
        aTable.getColumnModel().getColumn(1).setPreferredWidth(50);
        aTable.getColumnModel().getColumn(1).setMaxWidth(100);
        aTable.getColumnModel().getColumn(2).setMinWidth(100);
        aTable.getColumnModel().getColumn(2).setPreferredWidth(300);
        aTable.getColumnModel().getColumn(2).setMaxWidth(1000);
        aTable.setShowGrid(false);
        aTable.getTableHeader().setToolTipText("<html>Click table header to sort rows<br>ctrl-click to reverse sort order</html>");
        aTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        aTable.getTableHeader().addMouseListener(getTitleModel().aMouseHandler);
        aTable.setToolTipText("Double click to edit track or title");

        centerPane.getViewport().setView(aTable);
        southPanel.add(Box.createRigidArea(new Dimension(0, 3)));
        aButtonPanel = ComponentFactory.createPanelWithTwoColumns(1, 3);

        aTestRenameButton = ComponentFactory.createButton("&Test Rename Title", "", getTitleModel());
        aButtonPanel.add(aTestRenameButton);

        aUndoTestButton = ComponentFactory.createButton("&Undo Test", "", getTitleModel());
        aButtonPanel.add(aUndoTestButton);

        aUpdateButton = ComponentFactory.createButton("Write Title Tags", "", getTitleModel());
        aButtonPanel.add(aUpdateButton);

        southPanel.add(aButtonPanel);
        add(centerPane, BorderLayout.CENTER);
        add(southPanel, BorderLayout.SOUTH);

        aTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }

                ListSelectionModel lsm = (ListSelectionModel) e.getSource();

                if (lsm.isSelectionEmpty() == false) {
                    int selectedRow = lsm.getMinSelectionIndex();

                    JTagger.get().selectRow(selectedRow);
                }
            }
        });
    }


    /**
     *
     * @return Model object for table
     */
    public TitleModel getTitleModel() {
        return (TitleModel) aTable.getModel();
    }


    /**
     * Select row on grid.
     *
     * @param row Row number
     */
    public void selectRow(int row) {
        if (row >= 0) {
            ListSelectionModel lsm = aTable.getSelectionModel();
            lsm.setSelectionInterval(row, row);
        }
    }


    /**
     * Enable/disable buttons according to table status
     */
    public void updateButtons() {
        Component[] components = aButtonPanel.getComponents();
        for (Component component : components) {
            if (JTagger.get().getMainWindow().getTrackPane().getTrackTable().getTrackModel().getAudioTracks().size() > 0) {
                component.setEnabled(true);
            }
            else {
                component.setEnabled(false);
            }
        }
    }
}
